<?

class nc_payment_system_payanyway extends nc_payment_system {

    const ERROR_MNT_ID_IS_NOT_VALID = NETCAT_MODULE_PAYMENT_PAYANYWAY_ERROR_MNT_ID_IS_NOT_VALID;

    const TARGET_URL = "https://www.payanyway.ru/assistant.htm";

    protected $automatic = TRUE;

    // принимаемые валюты
    protected $accepted_currencies = array('RUB', 'RUR');

    // параметры сайта в платежной системе
    protected $settings = array(
        'MNT_ID' => null,
        'MNT_TEST_MODE' => null,
        'MNT_DATAINTEGRITY_CODE' => null,
        'MNT_KASSA_ENABLED' => null,
    );

    // передаваемые параметры
    protected $request_parameters = array( // 'InvId' => null,
        // 'InvDesc' => null,
    );

    // получаемые параметры
    protected $callback_response = array(
        'MNT_ID' => null,
        'MNT_TRANSACTION_ID' => null,
	    'MNT_OPERATION_ID' => null,
	    'MNT_TEST_MODE' => null,
		'MNT_AMOUNT' => null,
	    'MNT_CURRENCY_CODE' => null,
	    'MNT_SIGNATURE' => null,
    );

    /**
     *
     */
    public function execute_payment_request(nc_payment_invoice $invoice) {
        $amount = $invoice->get_amount("%0.2F");
	    $currency = ($invoice->get_currency() == 'RUR') ? 'RUB' : $invoice->get_currency();

        $signature_values = array(
            $this->get_setting('MNT_ID'),
	        $invoice->get_id(),
            $amount,
	        $currency,
	        $this->get_setting('MNT_TEST_MODE'),
            $this->get_setting('MNT_DATAINTEGRITY_CODE'),
        );
        $signature = md5(join("", $signature_values));

        ob_end_clean();
        $form = "
            <html>
              <body>
                    <form action='" . nc_payment_system_payanyway::TARGET_URL . "' method='post' accept-charset='utf-8'>" .
                    $this->make_inputs(array(
                        'MNT_ID' => $this->get_setting('MNT_ID'),
                        'MNT_TRANSACTION_ID' => $invoice->get_id(),
                        'MNT_AMOUNT' => $amount,
	                    'MNT_CURRENCY_CODE' => $currency,
	                    'MNT_TEST_MODE' => $this->get_setting('MNT_TEST_MODE'),
                        'MNT_DESCRIPTION' => $invoice->get_description(),
                        'MNT_SIGNATURE' => $signature,
                    )) . "
                </form>
                <script>
                  document.forms[0].submit();
                </script>
              </body>
            </html>";
        echo $form;
        exit;
    }

    /**
     * @param nc_payment_invoice $invoice
     */
    public function on_response(nc_payment_invoice $invoice = null) {

        $invoice->set('status', nc_payment_invoice::STATUS_SUCCESS);
        $invoice->save();
        $this->on_payment_success($invoice);
        echo 'SUCCESS';

        // print check via https://kassa.payanyway.ru
        if ($this->get_setting('MNT_KASSA_ENABLED')) {
            $inventoryPositions = array();
            $netshop = nc_netshop::get_instance();
            $order = $netshop->load_order($invoice->get('order_id'));

            $discount = $order->get_discount_sum();
            $sum = $invoice->get_amount();
            if ($discount) {
                $discountRate = ($sum - $discount) / $sum;
            }
            else {
                $discountRate = 1;
            }

            $items = $order->get_items();
            foreach ($items as $item) {
                $inventoryPositions[] = array('name' => $item['Name'], 'price' => round($item['OriginalPrice'] * $discountRate, 5), 'quantity' => $item['Qty'], 'vatTag' => 1105);
            }

            $shipping_sum = $order['DeliveryCost'] + $order['PaymentCost'];
            if ($shipping_sum) {
                $inventoryPositions[] = array('name' => 'Delivery', 'price' => round($shipping_sum * $discountRate, 5), 'quantity' => 1, 'vatTag' => 1105);
            }

            $data = array(
                'id' => $invoice->get_id(),
                'checkoutDateTime' => date(DATE_ATOM),
                'docNum' => $invoice->get_id(),
                'docType' => 'SALE',
                'email' => $invoice->get('customer_email'),
                'responseURL' => 'https://kassa.payanyway.ru/index.php?do=callback',
                'inventPositions' => $inventoryPositions,
                'moneyPositions' => array(array('paymentType' => 'CARD', 'sum' => $sum))
            );

            $data['signature'] = md5($data['id'].$data['checkoutDateTime'].$this->get_setting('MNT_DATAINTEGRITY_CODE'));
            $jsonData = json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
            $operationUrl = 'https://kassa.payanyway.ru/api/api.php?method=sale&accountid='.$this->get_response_value('MNT_ID');
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $operationUrl);
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $jsonData);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',
                'Content-Length: '.strlen($jsonData)
            ));
            $result = curl_exec($ch);
            curl_close($ch);

        }

        die();
    }

    /**
     *
     */
    public function validate_payment_request_parameters() {
        if (!$this->get_setting('MNT_ID')) {
            $this->add_error(nc_payment_system_payanyway::ERROR_MNT_ID_IS_NOT_VALID);
        }
    }

    /**
     * @param nc_payment_invoice $invoice
     */
    public function validate_payment_callback_response(nc_payment_invoice $invoice = null) {
        $their_key = strtoupper($this->get_response_value('MNT_SIGNATURE'));

        $signature_values = array(
            $this->get_response_value('MNT_ID'),
	        $this->get_response_value('MNT_TRANSACTION_ID'),
	        $this->get_response_value('MNT_OPERATION_ID'),
	        $this->get_response_value('MNT_AMOUNT'),
	        $this->get_response_value('MNT_CURRENCY_CODE'),
	        $this->get_response_value('MNT_TEST_MODE'),
	        $this->get_setting('MNT_DATAINTEGRITY_CODE')
        );
        $our_key = strtoupper(md5(join("", $signature_values)));

        if (!$invoice || $our_key != $their_key ) {
            if ($invoice) {
                $invoice->set('status', nc_payment_invoice::STATUS_CALLBACK_ERROR);
                $invoice->save();
            }
	        die('FAIL');
        }
    }

    public function load_invoice_on_callback() {
        return $this->load_invoice($this->get_response_value('MNT_TRANSACTION_ID'));
    }

    public function can_create_bill() {
        return ($this->get_setting('MNT_KASSA_ENABLED')) ? true : false;
    }

}

